package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Rajesh Danda
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.usermanagement.commands.*;
import com.ibm.commerce.sap.jca.util.*;
import com.ibm.commerce.util.ResourceBundleHandler;


/**
 * The new implementation class for PostUserRegistrationUpdateCmd
 * This invokes UpdateSAPUserRegistrationCmd to update the
 * user profile in SAP.
 */
  
public class SAPExtUserRegistrationUpdateCmdImpl extends PostUserRegistrationUpdateCmdImpl implements PostUserRegistrationUpdateCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.SAPExtUserRegistrationUpdateCmdImpl";
	protected TypedProperty requestProperties = null;
	protected TypedProperty responseProperties = null;
public TypedProperty getRequestProperties()
{
	return requestProperties;
}
public TypedProperty getResponseProperties()
{
	return responseProperties;
}
public void performExecute() throws com.ibm.commerce.exception.ECException {
    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    //Add the business logic here
    try {
        //register only if the profile type is C or parent organization is Default Organization
        String profileType =
            requestProperties.getString(ECUserConstants.EC_USER_PROFILETYPE, "");
        String parentId =
            requestProperties.getString(ECUserConstants.EC_PARENTMEMBERID, "");

        //if Parent Id is not available in request, retreive it using User Registry bean
        if (parentId.equals("")) {
            String userID = requestProperties.getString(ECUserConstants.EC_MEMBERID, null);
            //available from tools
            String logonID =
                requestProperties.getString(ECUserConstants.EC_UREG_LOGONID, null);

            // condition to fetch using logonId, when userid is null - 
            if (userID == null) {

                com.ibm.commerce.user.objects.UserRegistryAccessBean usrRegAccessBean =
                    new com.ibm.commerce.user.objects.UserRegistryAccessBean();
                usrRegAccessBean = usrRegAccessBean.findByUserLogonId(logonID);
                userID = usrRegAccessBean.getUserId();
            }

            com.ibm.commerce.user.objects.UserAccessBean usersBean =
                new com.ibm.commerce.user.objects.UserAccessBean();
            usersBean.setInitKey_MemberId(userID);
            usersBean.refreshCopyHelper();
            parentId = usersBean.getParentMemberId();
        }

        // Here -2000, refers to the Default Organization and any
        // customer having this as the parent is assumed to be a B2C user

        if (profileType.equals(ECUserConstants.EC_USER_PROFILE_CUSTOMER)
            || (parentId.trim().equals("-2000"))) {
            UpdateSAPUserRegistrationCmd userEntity =
                (UpdateSAPUserRegistrationCmd) com
                    .ibm
                    .commerce
                    .command
                    .CommandFactory
                    .createCommand(
                        com.ibm.commerce.sap.jca.commands.UpdateSAPUserRegistrationCmd.NAME,
                        getStoreId());
            userEntity.setRequestProperties(requestProperties);
            userEntity.setResponseProperties(responseProperties);
            userEntity.execute();

            userEntity.getRequestProperties();
            userEntity.getResponseProperties();
        }

    } catch (javax.ejb.CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
}
public void setRequestProperties(TypedProperty aRequestProperties)
{
	requestProperties = aRequestProperties;
}
public void setResponseProperties(TypedProperty aResponseProperties)
{
	responseProperties = aResponseProperties;
}
}
