package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD 	WHO    Description
// --------------------------------------------------------------------
// 020527 	Nagaveni Sastry
//
////////////////////////////////////////////////////////////////////////////////


import java.math.*;
import java.sql.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import java.util.Date; 

import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.order.commands.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.taxation.objects.*;

import com.ibm.commerce.util.ResourceBundleHandler;
import com.ibm.commerce.messaging.commands.*;

import com.ibm.commerce.sap.jca.commands.*;
import com.ibm.eis.sap.jca.generated.*;
import com.ibm.commerce.sap.jca.util.*;
import com.sap.rfc.*;
import com.sap.rfc.exception.*;
import com.ibm.connector2.sap.*;



/**
 * The new implementation class for ProcessOrderCmd.
 * Extends ProcessOrderCmdImpl to Create/Cancel an order 
 * in SAP depending on the user request.  
 */

public class SAPOrderCreateCmdImpl extends ProcessOrderCmdImpl implements ProcessOrderCmd {
	private SAPConnectionSpec connSpec =null;
	private SAPConnectionFactory connFactory = null;
	private SAPConnection connection = null;

	OrderCreateBapi cmd = new OrderCreateBapi();

	private final static String  CLASS_NAME             = "com.ibm.commerce.sap.jca.commands.SAPOrderCreateCmdImpl";
	private static String  istrInitFunc = "initialize";
	private static String  istrCreateOrderFunc  = "createOrder"       ;

//	Commented, Rajesh june 22	
//	private TypedProperty   requestProperties    = new TypedProperty();

	private TypedProperty   iRequestProperties   = new TypedProperty();
	
	public java.lang.String SalesDocNumber;
	public com.ibm.eis.sap.jca.generated.BapisdheduTable scheduleEx;
	public com.ibm.eis.sap.jca.generated.BapiitemexTable itemEx;
/* 
 * This method checks for the changes in price.shipping, taxes and quantities of orderitems.
 * It compares the results of OrderCreate with the results for OrderSimulate.
 */

private boolean checkForChange(OrderAccessBean anabOrder, String docNumber)
    throws ECException {

    final String methodName = "checkForChange";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    boolean confirm = true; // default : schedule lines are not changed
    try {

        Timestamp tmCurrentTime = com.ibm.commerce.utils.TimestampHelper.now();
        anabOrder.setLastUpdate(tmCurrentTime);
        anabOrder.setPlaceOrderTime(tmCurrentTime);
        anabOrder.commitCopyHelper();

        OrderItemAccessBean[] abOrderItemList = anabOrder.getOrderItems();

        // update order items
        boolean changeFlag = false; // default ; no change in schedule lines
        // loop for each item  to get the schedule dates
        // update each order item if the schedule dates are different
        for (int itemExIndex = 0;
            itemExIndex < getItemEx().getRowCount();
            itemExIndex++) {
            String itemNumber = null;
            String materialNumber = abOrderItemList[itemExIndex].getPartNumber();

            BigDecimal priceTotal =
                (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal1().doubleValue()))
                    .movePointLeft(2)
                    .setScale(2, BigDecimal.ROUND_UP);
            BigDecimal shipTotal =
                (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal4().doubleValue()))
                    .movePointLeft(2)
                    .setScale(2, BigDecimal.ROUND_UP);
            BigDecimal taxTotal =
                (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal5().doubleValue()))
                    .movePointLeft(2)
                    .setScale(2, BigDecimal.ROUND_UP);
            BigDecimal shiptaxTotal =
                (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal6().doubleValue()))
                    .movePointLeft(2)
                    .setScale(2, BigDecimal.ROUND_UP);		
                    
            double difference = 0.0;
            difference =
                abOrderItemList[itemExIndex].getTotalProductInEJBType().doubleValue()
                    - priceTotal.doubleValue();
            if (difference < 0)
                difference = difference * (-1);

            if (difference > 1) // to avoid very small differences
                {
                changeFlag = true;
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Price Information - Changed for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());

            } else {
                abOrderItemList[itemExIndex].setTotalProduct(priceTotal.toString());
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Price Information - No change for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            }

            difference = 0.0;
            difference =
                abOrderItemList[itemExIndex].getShippingChargeInEJBType().doubleValue()
                    - shipTotal.doubleValue();
            if (difference < 0)
                difference = difference * (-1);

            if (difference > 1) {
                changeFlag = true;
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Shipping Information - Changed for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            } else {
                abOrderItemList[itemExIndex].setShippingCharge(shipTotal.toString());
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Shipping Information - No change for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            }

            difference = 0.0;
            difference =
                abOrderItemList[itemExIndex].getTaxAmountInEJBType().doubleValue()
                    - taxTotal.doubleValue();
            if (difference < 0)
                difference = difference * (-1);

            if (difference > 1) {
                changeFlag = true;
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Tax Information - Changed for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            } else {
                abOrderItemList[itemExIndex].setTaxAmount(taxTotal.toString());
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "Tax Information - No change for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            }

            difference = 0.0;
            difference =
                abOrderItemList[itemExIndex].getShippingTaxAmountInEJBType().doubleValue()
                    - shiptaxTotal.doubleValue();
            if (difference < 0)
                difference = difference * (-1);

            if (difference > 1) {
                changeFlag = true;
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "ShippingTax Information - Changed for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            } else {
                abOrderItemList[itemExIndex].setShippingTaxAmount(shiptaxTotal.toString());
                ECTrace.trace(
                    ECTraceIdentifiers.COMPONENT_EXTERN,
                    CLASS_NAME,
                    methodName,
                    "ShippingTax Information - No change for orderitem "
                        + abOrderItemList[itemExIndex].getOrderItemId());
            }

            if ((getItemEx().getRow(itemExIndex).getMaterial().trim())
                .equalsIgnoreCase(materialNumber)) {
                for (int scheduleIndex = 0;
                    scheduleIndex < getScheduleEx().getRowCount();
                    scheduleIndex++) {

                    if ((getScheduleEx().getRow(scheduleIndex).getItmNumberString().trim())
                        .equalsIgnoreCase(
                            getItemEx().getRow(itemExIndex).getItmNumberString().trim())) {
                        if (getScheduleEx().getRow(scheduleIndex).getConfirQty().intValue() > 0) {
                            if (((abOrderItemList[itemExIndex].getPromisedAvailableTimeInEJBType())
                                .equals(
                                    new Timestamp(getScheduleEx().getRow(scheduleIndex).getMsDate().getTime())))
                                && (abOrderItemList[itemExIndex].getAllocatedQuantityInEJBType().intValue()
                                    == getScheduleEx().getRow(scheduleIndex).getConfirQty().intValue())) {
                                ECTrace.trace(
                                    ECTraceIdentifiers.COMPONENT_EXTERN,
                                    CLASS_NAME,
                                    methodName,
                                    "Schedule line information - No change for orderitem "
                                        + abOrderItemList[itemExIndex].getOrderItemId());
                            } else {
                                changeFlag = true;
                                ECTrace.trace(
                                    ECTraceIdentifiers.COMPONENT_EXTERN,
                                    CLASS_NAME,
                                    methodName,
                                    "Schedule line information - Changed for orderitem "
                                        + abOrderItemList[itemExIndex].getOrderItemId());
                            }
                        } // (if confirmedqty > 0)	
                    } // if 
                } //for scheduleIndex
            } // if

            abOrderItemList[itemExIndex].setStatus("P");
            abOrderItemList[itemExIndex].setLastUpdate(tmCurrentTime);
            abOrderItemList[itemExIndex].commitCopyHelper();
        } // for itemExIndex

        if (changeFlag) {
            confirm = false; // order has changed
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "confirm=false");
        } else {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "confirm=true");
        }

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (RemoteException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	     
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	     
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

	    
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
	    
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (Exception e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrCreateOrderFunc,
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    return confirm;

}
/**
 * Calls BAPI_SALESORDER_CREATEFROMDAT1 Bapi to 
 * create a sales order in Backend
 */
private void createOrder(OrderAccessBean abOrder)
    throws com.ibm.commerce.exception.ECException {

    ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrCreateOrderFunc);

    // Initialize SAP connection
    initialize();

    //  Create BAPI SALES ORDER SIMULATE
    BAPI_SALESORDER_CREATEFROMDAT1 order = new BAPI_SALESORDER_CREATEFROMDAT1();

    try {

        String doctype =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DOC_TYPE,
                PropertiesKeysForSAP.sapResourceBundle);
        String salesorg =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._SALES_ORGANIZATION,
                PropertiesKeysForSAP.sapResourceBundle);
        String distrchan =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DISTRIBUTION_CHANNEL,
                PropertiesKeysForSAP.sapResourceBundle);
        String division =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DIVISION,
                PropertiesKeysForSAP.sapResourceBundle);

        String plant =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PLANT,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleSH =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_SH,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleSP =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_SP,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleBP =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_BP,
                PropertiesKeysForSAP.sapResourceBundle);

        java.util.Date reqDate = new java.util.Date();
        java.util.Date purchDate = new java.util.Date();
        java.util.Date itemReqDate = new java.util.Date();

        String curr = abOrder.getCurrency();

        OrderItemAccessBean[] abOIArr = abOrder.getOrderItems();

        com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean shipMode =
            new com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean();
        shipMode.setInitKey_shippingModeId(abOIArr[0].getShippingModeId());
        shipMode.refreshCopyHelper();
        String incot1 = shipMode.getCarrier().trim();
        String incot2 = shipMode.getCode().trim();

        // set OrderHeaderIn
        BapisdheadStructure ordHeadIn = new BapisdheadStructure();

        ordHeadIn.setDocType(doctype);
        ordHeadIn.setSalesOrg(salesorg);
        ordHeadIn.setDistrChan(distrchan);
        ordHeadIn.setDivision(division);
        ordHeadIn.setReqDateH(reqDate);
        ordHeadIn.setPurchNo(abOrder.getOrderId());
        ordHeadIn.setPurchDate(purchDate);
        ordHeadIn.setIncoterms1(incot1);
        ordHeadIn.setIncoterms2(incot2);
        ordHeadIn.setCurrency(curr);
        order.setOrderHeaderIn(ordHeadIn);

        BapiiteminTable itemsInTable = new BapiiteminTable();
        BapipartnrTable partTable = new BapipartnrTable();

        String partnerNumber = null;
        if (getUser()
            .getProfileType()
            .equals(com.ibm.commerce.user.helpers.ECUserConstants.EC_USER_PROFILE_CUSTOMER)
            || (getUser().getParentMemberId().equals("-2000"))) {
            partnerNumber = getUser().getUserField1();
        } else {

            OrganizationAccessBean orgAB = new OrganizationAccessBean();
            orgAB.setInitKey_MemberId(getUser().getParentMemberId());
            orgAB.refreshCopyHelper();
            partnerNumber = orgAB.getOrgEntityField1();
        }

        // prepare for all items in the current order
        OrderItemAccessBean[] abOIArray = abOrder.getOrderItems();

        // create for all orderitems in the order
        for (int a = 0; a < abOIArray.length; a++) {
            //set OrderItemsIn
            BapiiteminTableRow itemsInTableRow = itemsInTable.createEmptyRow();
            itemsInTableRow.setItmNumberString(abOIArray[a].getOrderItemId());
            itemsInTableRow.setPoItmNo(abOIArray[a].getOrderItemId());
            itemsInTableRow.setMaterial(abOIArray[a].getPartNumber());
            itemsInTableRow.setPlant(plant);
            itemsInTableRow.setReqQtyString(
                abOIArray[a].getNeededQuantityInEJBType().intValue() + "000");
            itemsInTableRow.setReqDate(itemReqDate);

            String itemShipModeId = abOIArray[a].getShippingModeId();
            com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean shippingBean =
                new com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean();

            if ((itemShipModeId != null) && !(itemShipModeId.equals(""))) {
                shippingBean.setInitKey_shippingModeId(itemShipModeId);
                shippingBean.refreshCopyHelper();
                itemsInTableRow.setIncoterms1(shippingBean.getCarrier());
                itemsInTableRow.setIncoterms2(shippingBean.getCode());

            }

            itemsInTable.appendRow(itemsInTableRow);

            //partner function at orderitem level
            //set OrderPartners for ship-to "SH" if not null
            if ((partRoleSH != null) && !(partRoleSH.trim().equals(""))) {
                BapipartnrTableRow partTableRow = partTable.createEmptyRow();
                partTableRow.setPartnRole(partRoleSH);

                partTableRow.setPartnNumb(partnerNumber);
                partTableRow.setItmNumberString(abOIArray[a].getOrderItemId());

                AddressAccessBean shipAddrBean = new AddressAccessBean();
                shipAddrBean.setInitKey_AddressId(abOIArray[a].getAddressId());
                shipAddrBean.refreshCopyHelper();

                StringBuffer strBuffer = new StringBuffer();
                if ((shipAddrBean.getAddress1() != null)
                    && !(shipAddrBean.getAddress1().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress1().trim());
                if ((shipAddrBean.getAddress2() != null)
                    && !(shipAddrBean.getAddress2().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress2().trim());
                if ((shipAddrBean.getAddress3() != null)
                    && !(shipAddrBean.getAddress3().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress3().trim());

                if (strBuffer.length() > 35) {
                    partTableRow.setStreet(strBuffer.substring(0, 34));
                } else {
                    partTableRow.setStreet(strBuffer.toString());
                }

                if ((shipAddrBean.getCity() != null)
                    && !(shipAddrBean.getCity().trim().equals("")))
                    partTableRow.setCity(shipAddrBean.getCity().trim());
                if ((shipAddrBean.getCountry() != null)
                    && !(shipAddrBean.getCountry().trim().equals("")))
                    partTableRow.setCountry(shipAddrBean.getCountry().trim());

                //to get SAP statecode for a given wc state(s) 
                String stateAbbr = shipAddrBean.getState().trim();
                if ((stateAbbr != null) && !(stateAbbr.equals(""))) {

                    StateCodeAccessBean stateCode = new StateCodeAccessBean();
                    stateCode.setInitKey_strStateAbbr(stateAbbr);
                    stateCode.setInitKey_strProductVendor(
                        PropertiesKeysForSAP.sapResourceBundle.getString(
                            PropertiesKeysForSAP._PRODUCT_VENDOR));
                    stateCode.refreshCopyHelper();

                    partTableRow.setRegion(stateCode.getStateCode());
                }
                if ((shipAddrBean.getZipCode() != null)
                    && !(shipAddrBean.getZipCode().trim().equals("")))
                    partTableRow.setPostlCode(shipAddrBean.getZipCode());
                if ((shipAddrBean.getLastName() != null)
                    && !(shipAddrBean.getLastName().trim().equals("")))
                    partTableRow.setName(shipAddrBean.getLastName());

                partTable.appendRow(partTableRow);
            } // (if  partRoleSH != null)			

        } // order items

        //partner functions at order level

        //set OrderPartners for bill-to "BP"
        if ((partRoleBP != null) && !(partRoleBP.trim().equals(""))) {

            BapipartnrTableRow partTableRow = partTable.createEmptyRow();
            partTableRow.setPartnRole(partRoleBP);
            partTableRow.setPartnNumb(partnerNumber);

            AddressAccessBean billAddrBean = new AddressAccessBean();
            billAddrBean.setInitKey_AddressId(abOrder.getAddressId());
            billAddrBean.refreshCopyHelper();

            StringBuffer strBuffer = new StringBuffer();
            if ((billAddrBean.getAddress1() != null)
                && !(billAddrBean.getAddress1().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress1().trim());
            if ((billAddrBean.getAddress2() != null)
                && !(billAddrBean.getAddress2().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress2().trim());
            if ((billAddrBean.getAddress3() != null)
                && !(billAddrBean.getAddress3().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress3().trim());

            if ((strBuffer.length() > 0) && (strBuffer != null)) {
                if (strBuffer.length() > 35) {
                    partTableRow.setStreet(strBuffer.substring(0, 34));
                } else {
                    partTableRow.setStreet(strBuffer.toString());
                }
            }

            if ((billAddrBean.getCity() != null)
                && !(billAddrBean.getCity().trim().equals("")))
                partTableRow.setCity(billAddrBean.getCity().trim());
            if ((billAddrBean.getCountry() != null)
                && !(billAddrBean.getCountry().trim().equals("")))
                partTableRow.setCountry(billAddrBean.getCountry().trim());

            //to get SAP statecode for a given wc state(s) 
            String stateAbbr = billAddrBean.getState().trim();
            if ((stateAbbr != null) && !(stateAbbr.equals(""))) {

                StateCodeAccessBean stateCode = new StateCodeAccessBean();
                stateCode.setInitKey_strStateAbbr(stateAbbr);
                stateCode.setInitKey_strProductVendor(
                    PropertiesKeysForSAP.sapResourceBundle.getString(
                        PropertiesKeysForSAP._PRODUCT_VENDOR));
                stateCode.refreshCopyHelper();

                partTableRow.setRegion(stateCode.getStateCode());
            }
            if ((billAddrBean.getZipCode() != null)
                && !(billAddrBean.getZipCode().trim().equals("")))
                partTableRow.setPostlCode(billAddrBean.getZipCode());
            if ((billAddrBean.getLastName() != null)
                && !(billAddrBean.getLastName().trim().equals("")))
                partTableRow.setName(billAddrBean.getLastName());

            partTable.appendRow(partTableRow);
        } // if (partRoleBP != null)

        //set OrderPartners for sold-to "SP"
        if ((partRoleSP != null) && !(partRoleSP.trim().equals(""))) {
            BapipartnrTableRow partTableRow = partTable.createEmptyRow();
            partTableRow.setPartnRole(partRoleSP);

            partTableRow.setPartnNumb(partnerNumber);
            partTable.appendRow(partTableRow);
        } // if (partRoleSP !=0)

        order.setOrderItemsIn(itemsInTable);
        order.setOrderPartners(partTable);
        order.setConvertParvwAuart("X");

        cmd.setCeInput(order);

        cmd.execute();


        Bapireturn1Structure retStruct = cmd.getCeOutput0().getReturn();

        String retType = retStruct.getType();
        String retData = retStruct.getMessage();

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            istrCreateOrderFunc,
            "**SAP Return Type** " + retType);
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            istrCreateOrderFunc,
            "**SAP Return Data** " + retData);

        if (retType.equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrCreateOrderFunc,
                "**Failed to Create Order**");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_CREATE_ORDER,
                CLASS_NAME,
                istrCreateOrderFunc,
                "GenericApplicationError",
                true);

        } else {
            setSalesDocNumber(cmd.getCeOutput0().getSalesdocument());
            setScheduleEx(cmd.getCeOutput0().getOrderScheduleEx());
            setItemEx(cmd.getCeOutput0().getOrderItemsOut());

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrCreateOrderFunc,
                "** Order Created successfully**" + getSalesDocNumber());

        }

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    } catch (javax.ejb.FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            istrCreateOrderFunc,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);

    }
    
    ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrCreateOrderFunc);

}
/**
 * @return com.ibm.eis.sap.jca.generated.BapiitemexTable
 */
private com.ibm.eis.sap.jca.generated.BapiitemexTable getItemEx() {
	return itemEx;
}
/**
 * Gets the order reference number.
 */
public Long getOrderRn()
	{
		return super.getOrderRn();
	}
/**
 * Get request properties.
 * 
 * @return Request Properties
 */
 
public com.ibm.commerce.datatype.TypedProperty getRequestProperties() {
	return getCommandContext().getRequestProperties();
	
}
/**
 * @return java.lang.String
 */
private java.lang.String getSalesDocNumber() {
	return SalesDocNumber;
}
/**
 * @return com.ibm.eis.sap.jca.generated.BapisdheduTable
 */
private com.ibm.eis.sap.jca.generated.BapisdheduTable getScheduleEx() {
	return scheduleEx;
}
/**
 * Establishing the connection to SAP backend system
 */
 
 
protected void initialize() throws com.ibm.commerce.exception.ECException 
{
	ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrInitFunc);	


     try {


        // Setting properties at runtime


            Properties configProp = new Properties();

            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);

			String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
	
            com.ibm.connector2.sap.SAPConnectionFactory scf =
                (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

            com.ibm.connector2.sap.SAPConnectionSpec scs =
                new com.ibm.connector2.sap.SAPConnectionSpec();

            scs.setUserName(
                ResourceBundleHandler.getStringByKey(
                    PropertiesKeysForSAP._USERID,
                    PropertiesKeysForSAP.sapResourceBundle));
            scs.setPassword(
                ResourceBundleHandler.getStringByKey(
                    PropertiesKeysForSAP._PASSWD,
                    PropertiesKeysForSAP.sapResourceBundle));

            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) scf.getConnection(scs);
            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();

            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);

     } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);    
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);   
    } catch (Exception e)	{
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, istrInitFunc, e,true);
    }

	ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrInitFunc);
		
}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException {
    final String methodName = "performExecute";

    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    // orderConfirm parameter would decide whether confirmation of order is YES/NO/CHNAGE/CANCEL 
    // YES : Order confirmed(either changes are accpeted by user or no changes between
    //       ordersimulate and ordercreate (Order created in SAP and completed in WCS)
    // NO : Default value passed while invoking OrderProcess for the first time
    //      Order is not yet created in SAP and pending in WCS
    // CHANGE : This value is set if there is difference in schedule lines from the time of order prepare to order process
    // CANCEL : This values is passed from OrderConfirmView if user decides to cancel the order (marked for delete in SAP and cancelled in WCS)
    
    String orderConfirm = getRequestProperties().getString("orderConfirm");

    ECTrace.trace(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        methodName,
        "===OrderConfirm : " + orderConfirm + " Order not yet created in SAP ");

    String orderNumber = getOrderRn().toString();
    OrderAccessBean OAB = new OrderAccessBean();

    // orderConfirm would be NO for the first time by default so that
    // it creates the order in SAP and check for difference in 
    // schedule lines and price, tax and shipping information

    if (orderConfirm.trim().equals("NO")) {
        try {
            OAB.setInitKey_orderId(orderNumber);
            OAB.refreshCopyHelper();
            OAB.setStatus("P");
            OAB.commitCopyHelper();

            // call createorder() on SAP
              createOrder(OAB);

            // Insert SAP order number in ORDERS table
            // returns a boolean whether schedule lines are changed or not
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Order created on SAP - SAP Sales Order number : " + getSalesDocNumber());

            boolean confirm = checkForChange(OAB, getSalesDocNumber());

            if (confirm) {
                orderConfirm = "YES";
            } else {
                orderConfirm = "CHANGE";
            }

        } catch (RemoteException e) {
            throw new ECSystemException(
                ECMessage._ERR_REMOTE_EXCEPTION,
                CLASS_NAME,
                methodName,
                ECMessageHelper.generateMsgParms(e.getMessage()),
                e,
                true);
        } catch (CreateException e) {
            throw new ECSystemException(
                ECMessage._ERR_CREATE_EXCEPTION,
                CLASS_NAME,
                methodName,
                ECMessageHelper.generateMsgParms(e.getMessage()),
                e,
                true);
        } catch (NamingException e) {
            throw new ECSystemException(
                ECMessage._ERR_NAMING_EXCEPTION,
                CLASS_NAME,
                methodName,
                ECMessageHelper.generateMsgParms(e.getMessage()),
                e,
                true);
        } catch (FinderException e) {
            throw new ECSystemException(
                ECMessage._ERR_FINDER_EXCEPTION,
                CLASS_NAME,
                methodName,
                ECMessageHelper.generateMsgParms(e.getMessage()),
                e,
                true);
        }
    } // end of if (orderConfirm = NO)

    /* If the order is confirmed then,
    
     1. Status is changed from 'P' to 'C' by invoking super.performExecute()
    
     2. OrderStatusCmd is called to make entries in backend tables
        ORDSTAT and ORDISTAT tables with ORDERS and ORDERITEMS tables
        information respectively.Status is changed from 'C' to 'G' 
    */

    if (orderConfirm.trim().equals("YES")) {

        //Update OrderItem Details
        updateOrderItemDetails();

        // To Process the order in WC (status is changed from P to C)
        try {
            super.performExecute();
            
        } catch (ECSystemException e) {

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Failed to process Order in WebSphere Commerce for OrderID = "
                    + getOrderRn()
                    + ", SAP SalesDocument Number = "
                    + getSalesDocNumber());

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

            throw new ECSystemException(
                ECMessage._ERR_GENERIC,
                CLASS_NAME,
                methodName,
                ECMessageHelper.generateMsgParms(e.getMessage()),
                e,
                true);

        } catch (ECApplicationException e) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Failed to process Order in WebSphere Commerce for OrderID = "
                    + getOrderRn()
                    + ", SAP SalesDocument Number = "
                    + getSalesDocNumber());

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

            throw new ECApplicationException(
                ECMessage._ERR_GENERIC,
                CLASS_NAME,
                methodName,
                "GenericApplicationError",
                true);

        }

        //update Order totals and merchant order number
        updateOrderTotals();

        //update Order Status details
        updateOrderStatus();

    } // if (orderConfirm = YES)
    else
        if (orderConfirm.trim().equalsIgnoreCase("CHANGE")) {
            confirmOrderChange();
        } else
            if (orderConfirm.trim().equalsIgnoreCase("CANCEL")) {
                // user selects to cancel the order
                cancelOrder();

            }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}
/**
 * @param newItemEx com.ibm.eis.sap.jca.generated.BapiitemexTable
 */
private void setItemEx(com.ibm.eis.sap.jca.generated.BapiitemexTable newItemEx) {
	itemEx = newItemEx;
}
/**
 * @param newSalesDocNumber java.lang.String
 */
private void setSalesDocNumber(java.lang.String newSalesDocNumber) {
	SalesDocNumber = newSalesDocNumber;
}
/**
 * @param newScheduleEx com.ibm.eis.sap.jca.generated.BapisdheduTable
 */
private void setScheduleEx(com.ibm.eis.sap.jca.generated.BapisdheduTable newScheduleEx) {
	scheduleEx = newScheduleEx;
}

private void cancelOrder() throws com.ibm.commerce.exception.ECException {

    final String methodName = "cancelOrder";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    ECTrace.trace(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        methodName,
        "Order" + getOrderRn() + " to be Cancelled");

    String salesDocNumber =
        getRequestProperties().getString("salesDocNumber", null);
    if ((salesDocNumber != null) || !(salesDocNumber.trim().equals("")))
        setSalesDocNumber(salesDocNumber);

    try {
        OrderAccessBean tmpOAB = new OrderAccessBean();
        tmpOAB.setInitKey_orderId(getOrderRn().toString());
        tmpOAB.refreshCopyHelper();
        tmpOAB.setStatus("P");
        tmpOAB.setMerchantOrderId(getSalesDocNumber());
        tmpOAB.commitCopyHelper();

        // Orderchange Bapi is called to mark the order as delete on SAP
        SAPOrderCancel ordCancel = new SAPOrderCancel();
        ordCancel.initialize();
        ordCancel.cancelOrder(getSalesDocNumber());

        // OrderCancelCmd is called to cancel the order on WC
        OrderCancelCmd orderCancelCmd =
            (OrderCancelCmd) CommandFactory.createCommand(
                OrderCancelCmd.NAME,
                getCommandContext().getStore().getStoreEntityIdInEJBType());
        orderCancelCmd.setCommandContext(getCommandContext());

        TypedProperty requestProperties = new TypedProperty();
        requestProperties.put("orderId", getOrderRn().toString());
        //redirect to Main page of the store
        requestProperties.put("URL", "StoreCatalogDisplayView");

        orderCancelCmd.setRequestProperties(requestProperties);
        orderCancelCmd.execute();

        getResponseProperties().put(
            ECConstants.EC_VIEWTASKNAME,
            ECConstants.EC_GENERIC_REDIRECTVIEW);
        getResponseProperties().put(
            ECConstants.EC_REDIRECTURL,
            "StoreCatalogDisplayView");

    } catch (RemoteException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to cancel the order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Cancel the order in WebSphere Commerce");

        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (CreateException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to cancel the order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Cancel the order in WebSphere Commerce");

        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (NamingException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to cancel the order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Cancel the order in WebSphere Commerce");

        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (FinderException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to cancel the order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Cancel the order in WebSphere Commerce");

        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);

    } catch (Exception e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to cancel the order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Cancel the order in WebSphere Commerce");

        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            methodName,
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}

/*
 * Redirect to OrderConfirmView with the results of the OrderCreate call to SAP.
 * This method is called when there is a difference in
 * schedule lines or OrderCost between OrderCreate and OrderSimulate calls
 */
 
private void confirmOrderChange()
    throws com.ibm.commerce.exception.ECException {

    final String methodName = "confirmOrderChange";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    ECTrace.trace(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        methodName,
        "schedule lines changed : OrderConfirmView");

    // there is change in schedule lines
    // redirect to OrderConfirmView
    // user can either confirm the order or cancel the order 

    getResponseProperties().put(
        ECConstants.EC_VIEWTASKNAME,
        ECConstants.EC_GENERIC_REDIRECTVIEW);
    getResponseProperties().put(ECConstants.EC_REDIRECTURL, "OrderConfirmView");
    getResponseProperties().put("salesDocNumber", getSalesDocNumber());

    try {
        // put the schedule line information into request properties
        for (int itemExIndex = 0;
            itemExIndex < getItemEx().getRowCount();
            itemExIndex++) {
            String itemNumberString = getItemEx().getRow(itemExIndex).getItmNumberString();
            BigDecimal shippingItemTotal = new BigDecimal("0");
            BigDecimal taxItemTotal = new BigDecimal("0");
            BigDecimal shippingTaxTotal = new BigDecimal("0");
            double priceTotal = 0.0;
            BigDecimal unitPrice = new BigDecimal(0);

            if (getItemEx().getRow(itemExIndex).getSubtotal1().doubleValue() > 0) {
                priceTotal = getItemEx().getRow(itemExIndex).getSubtotal1().doubleValue();
                unitPrice =
                    (new BigDecimal(priceTotal
                        / (getItemEx().getRow(itemExIndex).getReqQty().doubleValue())))
                        .movePointLeft(2)
                        .setScale(2, BigDecimal.ROUND_UP);
            }

            if (getItemEx().getRow(itemExIndex).getSubtotal4().doubleValue() > 0) {
                shippingItemTotal =
                    (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal4().doubleValue()))
                        .movePointLeft(2)
                        .setScale(2, BigDecimal.ROUND_UP);
            }
            if (getItemEx().getRow(itemExIndex).getSubtotal5().doubleValue() > 0) {
                taxItemTotal =
                    (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal5().doubleValue()))
                        .movePointLeft(2)
                        .setScale(2, BigDecimal.ROUND_UP);
            }
            if (getItemEx().getRow(itemExIndex).getSubtotal6().doubleValue() > 0) {
                shippingTaxTotal =
                    (new BigDecimal(getItemEx().getRow(itemExIndex).getSubtotal6().doubleValue()))
                        .movePointLeft(2)
                        .setScale(2, BigDecimal.ROUND_UP);
            }

            getResponseProperties().put(
                itemNumberString.trim() + "_price_" + itemExIndex,
                unitPrice.toString());
            getResponseProperties().put(
                itemNumberString.trim() + "_pricetotal_" + itemExIndex,
                (new BigDecimal(priceTotal))
                    .movePointLeft(2)
                    .setScale(2, BigDecimal.ROUND_UP)
                    .toString());
            getResponseProperties().put(
                itemNumberString.trim() + "_ship_" + itemExIndex,
                shippingItemTotal.toString());
            getResponseProperties().put(
                itemNumberString.trim() + "_tax_" + itemExIndex,
                taxItemTotal.toString());
            getResponseProperties().put(
                itemNumberString.trim() + "_shiptax_" + itemExIndex,
                shippingTaxTotal.toString());

            getResponseProperties().put(
              itemNumberString.trim() + "_requestedqty_" + itemExIndex, String.valueOf(getItemEx().getRow(itemExIndex).getReqQty().intValue()));


            getResponseProperties().put(
                itemNumberString.trim() + "_scheduleCount_" + itemExIndex,
                String.valueOf(getScheduleEx().getRowCount()));

          
            for (int scheduleExIndex = 0;
                scheduleExIndex < getScheduleEx().getRowCount();
                scheduleExIndex++) {
                int newScheduleCount = 1;

                if ((getItemEx()
                    .getRow(itemExIndex)
                    .getItmNumberString()
                    .trim()
                    .equalsIgnoreCase(
                        getScheduleEx().getRow(scheduleExIndex).getItmNumberString()))) {
                    if (getScheduleEx().getRow(scheduleExIndex).getConfirQty().intValue() > 0) {

                        getResponseProperties().put(
                            itemNumberString.trim() + "_msdate_" + itemExIndex + "_" + scheduleExIndex,
                            String.valueOf(getScheduleEx().getRow(scheduleExIndex).getMsDate().getTime()));
                        getResponseProperties().put(
                            itemNumberString.trim() + "_confirmqty_" + itemExIndex + "_" + scheduleExIndex,
                            String.valueOf(
                                getScheduleEx().getRow(scheduleExIndex).getConfirQty().intValue()));

                        newScheduleCount++;

                    }
                }
            } // for ScheduleEx

        } // for itemEx
    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.toString()),
            e);

    } catch (Exception e) {
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.toString()),
            e);

    }

 

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}

/*
 * Updates the ORDERITEMS and ORDERS tables to set the results
 * returned by SAP during OrderCreate
 */

private void updateOrderItemDetails()
    throws com.ibm.commerce.exception.ECException {

    final String methodName = "updateOrderItemDetails";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
    try {
        OrderAccessBean OAB = new OrderAccessBean();
        OAB.setInitKey_orderId(getOrderRn().toString().trim());
        OAB.refreshCopyHelper();
        OAB.setStatus("P");
        OAB.commitCopyHelper();
        OrderItemAccessBean[] OIAB = OAB.getOrderItems();

        for (int a = 0; a < OIAB.length; a++) {

            // update the items with new values for date, qty, price, shipping, tax etc 
            //from SAP avaialable in request properties

            String itemPricePerUnit =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_price_" + a,
                    null);

            String itemAvailDate =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_msdate_" + a,
                    null);

            String shipAmt =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_ship_" + a,
                    null);

            String taxAmt =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_tax_" + a,
                    null);

            String shipTaxAmt =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_shiptax_" + a,
                    null);

            String itemTotals =
                getRequestProperties().getString(
                    OIAB[a].getOrderItemId().trim() + "_pricetotal_" + a,
                    null);

            if ((itemPricePerUnit != null) && !(itemPricePerUnit.trim().equals(""))) {

                OIAB[a].setPrice(itemPricePerUnit.trim());

            }
            if ((itemTotals != null) && !(itemTotals.trim().equals(""))) {
                OIAB[a].setTotalProduct(itemTotals.trim());

            }

            if ((shipAmt != null) && !(shipAmt.trim().equals(""))) {
                OIAB[a].setShippingCharge(shipAmt.trim());

            }
            if ((taxAmt != null) && !(taxAmt.trim().equals(""))) {
                OIAB[a].setTaxAmount(taxAmt.trim());

            }
            if ((shipTaxAmt != null) && !(shipTaxAmt.trim().equals(""))) {
                OIAB[a].setShippingTaxAmount(shipTaxAmt.toString());

            }

            if ((itemAvailDate != null) && !(itemAvailDate.trim().equals(""))) {

                Timestamp ItemAvailDate = new Timestamp(Long.parseLong(itemAvailDate));
                OIAB[a].setPromisedAvailableTime(ItemAvailDate);
                OIAB[a].setEstimatedAvailableTime(ItemAvailDate);
                OIAB[a].commitCopyHelper();

            }

        } //end for loop

    } catch (RemoteException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");

        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }
    
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}

/*
 * Updates the ORDSTAT and ORDISTAT tables 
 * calls OrderStatus command 
 */

private void updateOrderStatus()
    throws com.ibm.commerce.exception.ECException {

    final String methodName = "updateOrderStatus";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    Vector itemsVector = new Vector();
    TypedProperty hOIAB = new TypedProperty();
    //
    try {

            OrderAccessBean OAB = new OrderAccessBean();
            OAB.setInitKey_orderId(getOrderRn().toString());
            OAB.refreshCopyHelper();

        OrderItemAccessBean[] OIAB = OAB.getOrderItems();

        for (int a = 0; a < OIAB.length; a++) {
            hOIAB = new TypedProperty();
            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .ORDER_ITEM_ID,
                OIAB[a].getOrderItemId());
            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .MERCHANT_ITEM_NUMBER,
                OIAB[a].getOrderItemId());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .PART_NUMBER,
                OIAB[a].getPartNumber());

            //to get Qty measure
            //Get the Baseitem ID
            com.ibm.commerce.inventory.objects.ItemSpecificationAccessBean itmSpcBean =
                new com.ibm.commerce.inventory.objects.ItemSpecificationAccessBean();
            itmSpcBean.setInitKey_itemspcId(OIAB[a].getItemSpecId());
            itmSpcBean.refreshCopyHelper();

            //Get the qty measure from baseitem
            com.ibm.commerce.inventory.objects.BaseItemAccessBean baseItemBean =
                new com.ibm.commerce.inventory.objects.BaseItemAccessBean();
            baseItemBean.setInitKey_baseItemId(itmSpcBean.getBaseItemId());
            baseItemBean.refreshCopyHelper();

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .UNIT_OF_MEASURE,
                baseItemBean.getQuantityMeasure());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .ORDER_ITEM_STATUS,
                "A");

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .QUANTITY_REQUESTED,
                OIAB[a].getQuantity());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .CURRENCY,
                OIAB[a].getCurrency());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .QUANTITY_CONFIRMED,
                OIAB[a].getAllocatedQuantity());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .PRICE_TOTAL,
                OIAB[a].getTotalProduct());

            hOIAB.put(
                com
                    .ibm
                    .commerce
                    .orderstatus
                    .objimpl
                    .OrderFulfillmentItemStatusConstants
                    .UNIT_PRICE,
                OIAB[a].getPrice());

            if ((OIAB[a].getShippingCharge() != null)
                && !(OIAB[a].getShippingCharge().trim().equals("")))
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .SHIPPING_TOTAL,
                    OIAB[a].getShippingCharge());

            if ((OIAB[a].getTaxAmount() != null)
                && !(OIAB[a].getTaxAmount().trim().equals("")))
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .TAX_TOTAL,
                    OIAB[a].getTaxAmount());

            if (OIAB[a].getLastUpdate() != null)
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .PLACE_DATE_TIME,
                    (OIAB[a].getLastUpdate()).toString());
            if ((OIAB[a].getEstimatedAvailableTime() != null)
                && !(OIAB[a].getEstimatedAvailableTime().toString().trim().equals("")))
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .REQUEST_SHIP_DATE_TIME,
                    (OIAB[a].getEstimatedAvailableTime()).toString());
            else
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .REQUEST_SHIP_DATE_TIME,
                    (new Timestamp(new Date().getTime())).toString());

            if ((OIAB[a].getPromisedAvailableTime() != null)
                && !(OIAB[a].getPromisedAvailableTime().trim().equals("")))
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .SCHEDULE_SHIP_DATE_TIME,
                    OIAB[a].getPromisedAvailableTime().toString());
            else
                hOIAB.put(
                    com
                        .ibm
                        .commerce
                        .orderstatus
                        .objimpl
                        .OrderFulfillmentItemStatusConstants
                        .SCHEDULE_SHIP_DATE_TIME,
                    OIAB[a].getEstimatedAvailableTime().toString());

            itemsVector.addElement(hOIAB);

        } //end if Order items

        TypedProperty requestProperties = new TypedProperty();
       
        requestProperties.put("URL", "/");
        // URL is a mandatory parameter for OrderPrepare command
        requestProperties.put(
            com.ibm.commerce.orderstatus.objimpl.OrderFulfillmentStatusConstants.ORDER_ID,
            OAB.getOrderId());
        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .ORDER_STATUS,
            "A");

        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .MERCHANT_ORDER_NUMBER,
            getSalesDocNumber());

        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .PLACE_DATE_TIME,
            OAB.getPlaceOrderTime());
        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .LAST_UPDATE_TIME,
            OAB.getLastUpdate());
        requestProperties.put(
            com.ibm.commerce.orderstatus.objimpl.OrderFulfillmentStatusConstants.CURRENCY,
            OAB.getCurrency());
        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .SHIPPING_TOTAL,
            OAB.getTotalShippingCharge());
        requestProperties.put(
            com.ibm.commerce.orderstatus.objimpl.OrderFulfillmentStatusConstants.TAX_TOTAL,
            OAB.getTotalTax());
        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .SHIPPING_TAX_TOTAL,
            OAB.getTotalShippingTax());
        requestProperties.put(
            com
                .ibm
                .commerce
                .orderstatus
                .objimpl
                .OrderFulfillmentStatusConstants
                .PRICE_TOTAL,
            OAB.getTotalProductPrice());

        requestProperties.put("itemsVector", itemsVector);//to set OrderItem details to request

        
      //invoking OrderStatus Command - updates OrderStatus tables        
        OrderStatusCmd orderStatus =
            (OrderStatusCmd) CommandFactory.createCommand(
                OrderStatusCmd.NAME,
                getCommandContext().getStore().getStoreEntityIdInEJBType());
        orderStatus.setAccCheck(false);
        orderStatus.setCommandContext(getCommandContext());

        orderStatus.setRequestProperties(requestProperties);
        orderStatus.execute();

                ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                methodName,
                "Updated Order Status in WebSphere Commerce for OrderID = "
                    + getOrderRn());                

    } catch (RemoteException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }

        ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}

/*
 * Updates the ORDERS tables to set the SalesDocumentNumber and totals 
 * 
 */

private void updateOrderTotals()
    throws com.ibm.commerce.exception.ECException {

    final String methodName = "updateOrderTotals";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);

    try {
        // get the order
        String orderNumber = getOrderRn().toString();

        OrderAccessBean OAB = new OrderAccessBean();
        OAB.setInitKey_orderId(orderNumber);
        OAB.refreshCopyHelper();

        String salesDocNumber =
            getRequestProperties().getString("salesDocNumber", null);
        if ((salesDocNumber == null)
            || (salesDocNumber.trim().equals(""))
            || salesDocNumber.trim().equals("null")) {
        } else
            setSalesDocNumber(salesDocNumber);

        OAB.setMerchantOrderId(getSalesDocNumber());     

        OAB.commitCopyHelper();

    

    } catch (RemoteException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    	    
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	    
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Failed to process Order in WebSphere Commerce for OrderID = "
                + getOrderRn()
                + ", SAP SalesDocument Number = "
                + getSalesDocNumber());

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            methodName,
            "Action: Order created in SAP, cancel the order in SAP by calling the order cancel BAPI. If it fails to cancel, manually delete the order in SAP");
	     
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }
    
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, methodName);
}

}
