package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Rajesh Danda
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.taxation.objects.*;
import com.ibm.commerce.util.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.usermanagement.commands.*;

import com.ibm.commerce.sap.jca.util.*;
import com.ibm.eis.sap.jca.generated.*;

import javax.naming.*;
import java.util.*;

/**
 * The implementation class for UpdateSAPOrgEntityCmd.
 * Used to update the orgentity profile in SAP.
 * called during post orgentity process
 */
 
public class UpdateSAPOrgEntityCmdImpl extends TaskCommandImpl implements UpdateSAPOrgEntityCmd
{

	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.UpdateSAPOrgEntityCmdImpl";
	protected TypedProperty requestProperties = null;
	protected TypedProperty responseProperties = null;

	CustomerChangeBapi cmd = new CustomerChangeBapi();

public TypedProperty getRequestProperties()
{
	return requestProperties;
}
public TypedProperty getResponseProperties()
{
	return responseProperties;
}

 /**
 * Establishing the connection to SAP backend system
 */

 protected void initialize() throws com.ibm.commerce.exception.ECException {

	final String METHODNAME = "initialize";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    try{

        // Setting properties at runtime


            Properties configProp = new Properties();

            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);


			String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
	
            com.ibm.connector2.sap.SAPConnectionFactory scf =
                (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

            com.ibm.connector2.sap.SAPConnectionSpec scs =
                new com.ibm.connector2.sap.SAPConnectionSpec();

  				scs.setUserName(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._USERID));
				scs.setPassword(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PASSWD));
				 
            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) scf.getConnection(scs);
            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();

            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);

     } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);        
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);
    } catch (Exception e) {
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASSNAME, METHODNAME, e,true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
  
	}
/**
 * Contains the actual business logic of the command.
 */

public void performExecute() throws com.ibm.commerce.exception.ECException {

    //Add the business logic here
    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    initialize();

    try {

        //  Change BAPI_CUSTOMER_CHANGEFROMDATA
        com.ibm.eis.sap.jca.generated.BAPI_CUSTOMER_CHANGEFROMDATA custCreate =
            new com.ibm.eis.sap.jca.generated.BAPI_CUSTOMER_CHANGEFROMDATA();

        String orgEntityID =
            requestProperties.getString(ECUserConstants.EC_ORG_ORGENTITYID, null);
        //available from tools      

        com.ibm.commerce.user.objects.OrganizationAccessBean orgBean =
            new com.ibm.commerce.user.objects.OrganizationAccessBean();
        orgBean.setInitKey_MemberId(orgEntityID);
        orgBean.refreshCopyHelper();
        String refNumber = orgBean.getOrgEntityField1();

        AddressAccessBean currAddress = null;

        // set PI_ADDRESS
        com.ibm.eis.sap.jca.generated.Bapikna101Structure piAddr =
            new com.ibm.eis.sap.jca.generated.Bapikna101Structure();

        String orgEntityName =
            requestProperties.getString(ECUserConstants.EC_ORG_ORGENTITYNAME, null);

        if (orgEntityName != null)
            piAddr.setName(orgEntityName.trim());
        else
            if (orgBean.getOrganizationName() != null) {
                piAddr.setName(orgBean.getOrganizationName());
                orgEntityName = orgBean.getOrganizationName();
            }

        String street1 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS1, null);
        String street2 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS2, null);
        String street3 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS3, null);

        AddressAccessBean addrBean = new AddressAccessBean();
        currAddress = addrBean.findByNickname(orgEntityName, (new Long(orgEntityID)));
        if (currAddress == null) {
            // throw an exception
            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_PRIMARY_ADDRESS,
                CLASSNAME,
                METHODNAME,
                ECMessageHelper.generateMsgParms(orgEntityID));
        }

        StringBuffer strBuffer = new StringBuffer();
        if ((street1 != null) && !(street1.trim().equals("")))
            strBuffer.append(street1 + ",");
        else
            if (currAddress.getAddress1() != null)
                strBuffer.append(currAddress.getAddress1().trim());

        if ((street2 != null) && !(street2.trim().equals("")))
            strBuffer.append(street2 + ",");
        else
            if (currAddress.getAddress2() != null)
                strBuffer.append(currAddress.getAddress2().trim());

        if ((street3 != null) && !(street3.trim().equals("")))
            strBuffer.append(street3);
        else
            if (currAddress.getAddress3() != null)
                strBuffer.append(currAddress.getAddress3().trim());

        if (strBuffer.length() > 35) {
            piAddr.setStreet(strBuffer.substring(0, 34));
        } else {
            piAddr.setStreet(strBuffer.toString());
        }

        String postalcode =
            requestProperties.getString(ECUserConstants.EC_ADDR_ZIPCODE, null);
        if (postalcode != null)
            piAddr.setPostlCode(postalcode);
        else
            if (currAddress.getZipCode() != null)
                piAddr.setPostlCode(currAddress.getZipCode());

        String city = requestProperties.getString(ECUserConstants.EC_ADDR_CITY, null);
        if (city != null)
            piAddr.setCity(city);
        else
            if (currAddress.getCity() != null)
                piAddr.setCity(currAddress.getCity());

        String lang = requestProperties.getString("defaultLang", null);
        if (lang == null) {
            lang = "-1";
        }

        String altLang = "SAPLANG_" + lang.substring(1, lang.length());
        piAddr.setLangu(PropertiesKeysForSAP.sapResourceBundle.getString(altLang));

        //to get SAP statecode for a given wc state(s) 
        String stateAbbr =
            requestProperties.getString(ECUserConstants.EC_ADDR_STATE, null);
        if (stateAbbr == null) {
            stateAbbr = currAddress.getState();
        }

        StateCodeAccessBean stateCode = new StateCodeAccessBean();
        stateCode.setInitKey_strStateAbbr(stateAbbr);
        stateCode.setInitKey_strProductVendor(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._PRODUCT_VENDOR));
        stateCode.refreshCopyHelper();
        piAddr.setRegion(stateCode.getStateCode());

        String country =
            requestProperties.getString(ECUserConstants.EC_ADDR_COUNTRY, null);

        if (country != null)
            piAddr.setCountry(country);
        else
            // both SAP and WCS follow same codes
            piAddr.setCountry(currAddress.getCountry());

        // not available in the data entered from Adminconsole, available in store when a self-registration is done by by user    
        String curr =
            requestProperties.getString(ECUserConstants.EC_USER_PREFERREDCURRENCY, null);
        if (curr != null)
            // otherwise do nothing, SAP will take based on country field
            piAddr.setCurrency(curr);

        custCreate.setPiAddress(piAddr);

        custCreate.setPiSalesorg(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._SALES_ORGANIZATION));
        custCreate.setPiDistrChan(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._DISTRIBUTION_CHANNEL));
        custCreate.setPiDivision(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._DIVISION));

        custCreate.setCustomerno(refNumber);

        // Set command params
        cmd.setCeInput(custCreate);
        cmd.execute();

        BapireturnStructure retStruct = cmd.getCeOutput0().getReturn();
        String retType = retStruct.getType();
        String retData = retStruct.getMessage();

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Type** " + retType);
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Data** " + retData);

        com.ibm.eis.sap.jca.generated.Bapikna101Structure peAddress =
            cmd.getCeOutput0().getPeAddress();
        String verifyCustomerEntry = peAddress.getName();

        if (retType.equals("E") || verifyCustomerEntry == null) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Failed to update SAP Customer Record**");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_CUSTOMER_CHANGE,
                CLASSNAME,
                METHODNAME,
                true);

        }

    } catch (javax.ejb.CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

}
public void setRequestProperties(TypedProperty aRequestProperties)
{
	requestProperties = aRequestProperties;
}
public void setResponseProperties(TypedProperty aResponseProperties)
{
	responseProperties = aResponseProperties;
}
}
